import { Factory } from "./factory.js";
import { Item } from "./thing.js";
import { findNextUrl, getLoggerFor } from "./utils.js";
const logger = getLoggerFor("template");
export class OmekaTemplates {
    config;
    templates = {};
    partial_loaded = false;
    partial_templates = {};
    properties = {};
    items = {};
    constructor(config) {
        this.config = config;
    }
    async preload_property(url) {
        logger.info("Preloading properties: " + url);
        const resp = await this.config.fetch_f(url);
        const next = findNextUrl(resp);
        const promise = next ? this.preload_property(next) : Promise.resolve();
        const json = await resp.json();
        for (const prop of json) {
            this.properties[prop["o:id"]] = prop;
        }
        await promise;
    }
    async preload_properties() {
        await this.preload_property(this.config.url("properties"));
    }
    async get_property(id) {
        logger.debug("Get property " + id);
        if (this.properties[id])
            return this.properties[id];
        const resp = await this.config.get(`properties/${id}`);
        const prop = await resp.json();
        this.properties[prop["o:id"]] = prop;
        return prop;
    }
    async transform_template(temp) {
        const classObj = temp["o:resource_class"];
        const template = {
            id: temp["o:id"],
            label: temp["o:label"],
            class: classObj ? classObj["o:id"] : undefined,
            properties: [],
        };
        for (const prop of temp["o:resource_template_property"]) {
            logger.info("Finding property " + JSON.stringify(prop));
            const newProp = await this.get_property(prop["o:property"]["o:id"]);
            newProp.dataTypes = prop["o:data_type"];
            template.properties.push(newProp);
        }
        return template;
    }
    async preload_partial_templates() {
        this.partial_loaded = true;
        let url = this.config.url("resource_templates");
        while (!!url) {
            const resp = await this.config.fetch_f(url);
            url = findNextUrl(resp);
            const json = await resp.json();
            for (const temp of json) {
                this.partial_templates[temp["o:id"]] = temp;
            }
        }
    }
    async preload_templates() {
        this.partial_loaded = true;
        logger.info("Preloading templates");
        let url = this.config.url("resource_templates");
        while (!!url) {
            const resp = await this.config.fetch_f(url);
            url = findNextUrl(resp);
            const json = await resp.json();
            for (const temp of json) {
                this.partial_templates[temp["o:id"]] = temp;
                logger.info("Preloading template " + JSON.stringify(temp));
                const template = await this.transform_template(temp);
                this.templates[template.id] = template;
            }
        }
    }
    async get_template(id) {
        logger.info("Get template " + id);
        if (this.templates[id])
            return this.templates[id];
        const resp = await this.config.get(`resource_templates/${id}`);
        const temp = await resp.json();
        const template = await this.transform_template(temp);
        this.templates[template.id] = template;
        return template;
    }
    async get_template_by_name(name) {
        if (!this.partial_loaded) {
            throw "Can only get template by name if the templates are at least partially loaded";
        }
        for (const partial of Object.values(this.partial_templates)) {
            if (partial["o:label"] == name) {
                return await this.get_template(partial["o:id"]);
            }
        }
    }
    async get_item(id, deep = false) {
        if (this.items[id])
            return this.items[id];
        const resp = await this.config.get(`items/${id}`);
        const json = await resp.json();
        const item = this.set_item(new Item("", id, undefined, json, {}));
        const template = await this.get_template(json["o:resource_template"]["o:id"]);
        const factory = new Factory(template, this.config);
        await factory.fill_item(json, deep, this, item);
        return item;
    }
    async get_items_set(item_set) {
        const items = [];
        let url = this.config.url("items", {
            item_set_id: item_set,
        });
        while (!!url) {
            logger.debug("Fetching url " + url);
            const resp = await this.config.fetch_f(url);
            url = findNextUrl(resp);
            const things = await resp.json();
            items.push(...things);
        }
        const factories = {};
        const loadable = [];
        const out = [];
        for (const thing of items) {
            const id = thing["o:id"];
            const item = this.set_item(new Item("", id, undefined, thing, {}));
            out.push(item);
            const template_id = thing["o:resource_template"]["o:id"];
            if (factories[template_id]) {
                loadable.push({ item, factory: factories[template_id], thing });
            }
            else {
                logger.debug("Creating factory for template " + template_id);
                const template = await this.get_template(template_id);
                const factory = new Factory(template, this.config);
                factories[template_id] = factory;
                loadable.push({ item, factory, thing });
            }
        }
        logger.debug("Filling " + loadable.length + "items");
        for (const l of loadable) {
            await l.factory.fill_item(l.thing, true, this, l.item);
        }
        return out;
    }
    set_item(item) {
        if (this.items[item.idx])
            return this.items[item.idx];
        return (this.items[item.idx] = item);
    }
}
